#!/usr/bin/env python
# Copyright (c) 2011-2012  The Board of Trustees of The Leland Stanford Junior University
# Copyright (c) 2012-2017  Barnstormer Softworks, Ltd.

import sys
import os
import shutil
from optparse import OptionParser
import subprocess

PIP = ""
PIP_PATHS = ["/usr/local/bin/pip", "/usr/bin/pip"]

PIP_PACKAGES = [
  "Werkzeug==0.12.2",
  "Jinja2==2.10",
  "itsdangerous==0.24",
  "SQLAlchemy==1.1.14",
  "lxml==4.0.0",
  "pyOpenSSL==17.3.0",
  "psutil==5.2.2",
  "cffi==1.11.2",
  "signxml==2.4.0",
  "boto3==1.4.7",
  "dropbox==8.4.1",
  "cryptography==2.0",
  "requests==2.18.4",
  "python-hglib==2.4"
]

def output (data):
  if data.strip():
    print data

def call (cmd, show_cmd = True, show_output = True):
  if show_cmd:
    print cmd
  p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
  (sout, serr) = p.communicate()
  if show_output:
    output(sout)
    output(serr)
  return sout

def symlink (src, dst):
  print "Creating symlink to %s from %s" % (src, dst)
  try:
    os.symlink(src, dst)
  except OSError, e:
    "Already exists!"

def copyfile (src, dst):
  print "Copying %s to %s" % (src, dst)
  shutil.copyfile(src, dst)

def fixup_version (opts):
  rev = call('hg id -i').strip()
  branch = call('hg id -b').strip()

  f = open("setup.py", "r")
  new_setup = []
  for line in f.readlines():
    if line.count("version"):
      new_setup.append("      version='%s (%s)',\n" % (rev, branch))
    else:
      new_setup.append(line)
  f.close()
  f = open("setup.py", "w+")
  f.write("".join(new_setup))

  f = open("src/foam/version.py", "w+")
  f.write("VERSION = '%s-%s'\n" % (rev, branch))
  f.close()

def addDir (path, owner):
  try:
    print "Making Directory: %s" % (path)
    os.makedirs(path)
  except OSError, e:
    pass
  call('chown %s %s' % (owner, path))

def install_foam ():
  call("python setup.py install")

  addDir("/opt/foam/db", "www-data")
  addDir("/opt/foam/etc/", "www-data")
  addDir("/opt/foam/etc/gcf-ca-certs", "root")
  addDir("/opt/foam/log", "www-data")
  addDir("/opt/foam/log/tasks", "www-data")
  addDir("/opt/foam/mounts", "www-data")
  addDir("/opt/foam/tasks/queue", "www-data")
  addDir("/opt/foam/tasks/completed", "www-data")
  addDir("/opt/foam/etc/templates/custom", "root")

def install_deps_ubuntu ():
  call("apt-get --yes install nginx xmlsec1 python-dateutil m2crypto python-dev ssl-cert libxml2-dev libxslt1-dev libssl-dev libffi-dev libzmq3-dev")


def install_deps (opts):
#  res = call("pip --version")
#  ver = res.split()[1]
#  if ver != "1.2.1":
#    call("pip install -U pip")

  if opts.platform == "ubuntu":
    install_deps_ubuntu()

  call("%s install -U %s" % (PIP, " ".join(PIP_PACKAGES)))

def postinst_ubuntu ():
  call("/usr/sbin/make-ssl-cert generate-default-snakeoil")
  copyfile("/etc/ssl/certs/ssl-cert-snakeoil.pem", "/opt/foam/etc/server.crt")
  copyfile("/etc/ssl/private/ssl-cert-snakeoil.key", "/opt/foam/etc/server.key")
  symlink("/etc/nginx/sites-available/foam.conf", "/etc/nginx/sites-enabled/foam.conf")
  copyfile("%s/debian/init.d" % (os.getcwd()), "/etc/init.d/foam")
  call("groupadd rdocker")

def install_sudo ():
  p = "%s/src/scripts/" % (os.getcwd())
  for fname in os.listdir(p):
    if fname.endswith(".sudo"):
      copyfile("%s/%s" % (p, fname), "/etc/sudoers.d/%s" % (fname.split(".")[0]))
  call("chmod 440 /etc/sudoers.d/*")

def postinst (opts):
  if opts.platform == "ubuntu":
    postinst_ubuntu()
  install_sudo()
  call("chmod +x /etc/init.d/foam")
  # Maybe ask if you really want to revert setup.py right now?
  call("hg revert setup.py src/foam/version.py --no-backup")

def parse_args (argv):
  parser = OptionParser()
  parser.add_option("--allow-uncommitted", dest="allow_uncommitted", default=False, action="store_true")
  parser.add_option("--platform", dest="platform", default="ubuntu")
  (opts, args) = parser.parse_args()
  return opts

def main ():
  if os.getuid() != 0:
    print "You must be root to run this installer."
    sys.exit(1)

  opts = parse_args(sys.argv)
  if not opts.allow_uncommitted:
    if call("hg status", False, False).strip():
      print "You must commit all changes before running this installer."
      sys.exit(1)

  global PIP
  for path in PIP_PATHS:
    if os.path.exists(path):
      PIP = path

  if not PIP:
    print "ERROR: pip not found, exiting."
    sys.exit(1)

  fixup_version(opts)
  install_foam()
  install_deps(opts)
  postinst(opts)

if __name__ == '__main__':
  main()
