#!/bin/sh
#
# foam GENI OpenFlow Aggregate Manager
#
# chkconfig:   2345 20 80
# description: GENI OpenFlow Aggregate Manager

### BEGIN INIT INFO
# Provides: foam
# Required-Start: $local_fs $network
# Required-Stop: 
# Should-Start: 
# Should-Stop: 
# Default-Start: 2 3 4 5
# Default-Stop: 0 1 6
# Short-Description: GENI OpenFlow Aggregate Manager
# Description: GENI OpenFlow Aggregate Manager
### END INIT INFO

# Source function library.
. /etc/rc.d/init.d/functions

fcgi="/opt/foam/sbin/foam.fcgi"
exec="python /opt/foam/sbin/foam.fcgi"
prog="foam"
pidfile="/var/run/foam.pid"
user="nginx"

[ -e /etc/sysconfig/$prog ] && . /etc/sysconfig/$prog

lockfile=/var/lock/subsys/$prog

log_success_msg () {
    echo -n $prog; success "$*"; echo
}

start() {
    [ -x $fcgi ] || exit 5
    echo -n $"Starting $prog: "
    daemon --check $prog --user $user --pidfile $pidfile "env - PYTHONPATH=/opt/foam/lib $exec &"
    retval=$?
    echo
    [ $retval -eq 0 ] && touch $lockfile
    echo "`pidof -x $exec`" > $pidfile
    return $retval
}

stop() {
    echo -n $"Stopping $prog: "
    if [ ! -f $pidfile ]; then
        log_success_msg
        return 0
    fi
    killproc -p $pidfile $exec
    retval=$?
    echo
    if [ $retval -eq 0 ] ; then
        rm -f $lockfile
        rm -f $pidfile
    fi
    return $retval
}

restart() {
    stop
    start
}

reload() {
    restart
}

force_reload() {
    restart
}

rh_status() {
    # run checks to determine if the service is running or use generic status
    status -p $pidfile $prog
}

rh_status_q() {
    rh_status >/dev/null 2>&1
}


case "$1" in
    start)
        rh_status_q && exit 0
        $1
        ;;
    stop)
        rh_status_q || exit 0
        $1
        ;;
    restart)
        $1
        ;;
    reload)
        rh_status_q || exit 7
        $1
        ;;
    force-reload)
        force_reload
        ;;
    status)
        rh_status
        ;;
    condrestart|try-restart)
        rh_status_q || exit 0
        restart
        ;;
    *)
        echo $"Usage: $0 {start|stop|status|restart|condrestart|try-restart|reload|force-reload}"
        exit 2
esac
exit $?
