# Conyright (c) 2016-2017  Barnstormer Softworks, Ltd.

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from distutils.core import setup
import os.path
import os
import pprint
import glob
import fnmatch

def generatePluginFiles ():
  pfiles = []
  foamroot = "/opt/foam"
  pdirs = os.listdir(os.getcwd() + "/plugins")
  for pd in pdirs:
    dl = []
    for plgfile in os.listdir(os.getcwd() + "/plugins/%s/" % (pd)):
      dl.append("plugins/%s/%s" % (pd, plgfile))
    pfiles.append(("%s/plugins/%s" % (foamroot, pd), dl))
  return pfiles

def generateDataDir (cur_dir_loc):
  files = []
  for f in os.listdir(os.getcwd() + "/%s" % (cur_dir_loc)):
    files.append("%s/%s" % (cur_dir_loc, f))
  return files

def opj (*args):
  # Handy utility from Robin Dunn
  path = os.path.join(*args)
  return os.path.normpath(path)

# distutils should include this function...
def generateDataFiles (srcdir, *wildcards, **kw):
  def walkHelper (arg, dirname, files):
    #print dirname
    names = []
    lst, wildcards, newroot, srclen = arg
    for wc in wildcards:
      wc_name = opj(dirname, wc)
      for f in files:
        filename = opj(dirname, f)
        if fnmatch.fnmatch(filename, wc_name) and not os.path.isdir(filename):
          names.append(filename)
    if names:
      lst.append(("%s/%s" % (newroot, dirname[srclen:]), names))

  file_list = []
  recursive = kw.get('recursive', True)
  newroot = kw.get('newroot', srcdir)
  if recursive:
    os.path.walk(srcdir, walkHelper, (file_list, wildcards, newroot, len(srcdir)))
  else:
    walkHelper((file_list, wildcards, newroot, len(srcdir)), srcdir,
               [os.path.basename(f) for f in glob.glob(opj(srcdir, '*'))])
  return file_list


def main ():
  data_files=[('/opt/foam/sbin', ['src/scripts/foam.fcgi']),
              ('/opt/foam/bin', ['src/scripts/expire', 'src/scripts/expire-emails',
                                 'src/scripts/daily-queue', 'src/scripts/logevent',
                                 'src/scripts/dropbox-upload', 'src/scripts/hg-worker',
                                 'src/scripts/destroy-tree']),
              ('/opt/foam/schemas', ['schemas/ad.xsd', 'schemas/request.xsd', 'schemas/of-resv-3.xsd',
                                     'schemas/any-extension-schema.xsd', 'schemas/request-common.xsd',
                                     'schemas/of-resv-4.xsd']),
              ('/etc/rsyslog.d/', ['src/scripts/58-vtsevent.conf']),
              ('/etc/nginx/sites-available/', ['src/foam.conf'])]
  data_files.extend(generatePluginFiles())
  data_files.extend(generateDataFiles('src/foamext/', "*.py", newroot='/opt/foam/lib/foamext'))
  data_files.extend(generateDataFiles('src/ext/', "*.py", newroot='/opt/foam/lib'))
  data_files.extend(generateDataFiles('src/foam/', "*.py", newroot='/opt/foam/lib/foam'))
  data_files.extend(generateDataFiles('templates/', "*.txt", newroot='/opt/foam/etc/templates/default'))
#  pprint.pprint(data_files)

  setup(name='foam',
      version='0.14.0',
        description='Virtual Topology Service Aggregate Manager',
        author='Nick Bastin',
        author_email='nick.bastin@gmail.com',
        url='https://openflow.stanford.edu/display/FOAM/Home',
        packages=[],
        package_dir={'foam': 'src/foam',
                     'jsonrpc' : 'src/ext/jsonrpc',
                     'sfa' : 'src/ext/sfa',
                     'geni' : 'src/ext/geni'},
        scripts=['src/scripts/foamctl'],
        data_files = data_files,
        install_requires = [
          "werkzeug",
          "jinja2",
          "itsdangerous",
          "sqlalchemy",
          "lxml",
          "pyOpenSSL",
          "psutil",
          "signxml",
          "cffi",
          "cryptography",
          "pyzmq",
          "isc_dhcp_leases",
          "python-hglib"
          ]
       )

if __name__ == '__main__':
  main()
