# Copyright (c) 2016  Barnstormer Softworks, Ltd.

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

import sys
import traceback

import boto3

from ..core import json
from ..core.configdb import ConfigDB, ConfigItem
from ..core.exception import CoreException

class _Publisher(object):
  def __init__ (self):
    self._client = None
    self._url = None
    self.site_tag = None

  def setup (self, akid, sak, region, queue_url):
    self._client = boto3.client("sqs", region, aws_access_key_id = akid, aws_secret_access_key = sak)
    self._url = queue_url

  def publishEvent (self, action, action_data, target_urn = None, user_urn = None, req_ip = None):
    if not self._client:
      return

    obj = {"v" : 2, "site" : self.site_tag, "action" : action, "action-data" : action_data}
    if target_urn: obj["target-urn"] = target_urn
    if user_urn: obj["user-urn"] = user_urn
    if req_ip: obj["req-ip"] = req_ip
    self._client.send_message(QueueUrl = self._url, MessageBody = json.dumps(obj))

  def publishError (self, exc, log = None):
    if not self._client:
      return

    try:
      obj = {"v" : 2, "site" : self.site_tag, "exc-msg" : str(exc)}
      if isinstance(exc, CoreException):
        if exc._unexpected:
          (typ, ex, tb) = sys.exc_info()
          obj["action"] = "unhandled-exception"
          obj["exc-tb"] = traceback.extract_tb(tb)
        else:
          obj["action"] = "exception"
      else:
        (typ, ex, tb) = sys.exc_info()
        obj["action"] = "unhandled-exception"
        obj["exc-tb"] = traceback.extract_tb(tb)

      self._client.send_message(QueueUrl = self._url, MessageBody = json.dumps(obj))
    except Exception, e:
      if log:
        log.exception("AWSEvent")
      

Publisher = _Publisher()

def setup (app):
  citems = []
  citems.append(ConfigItem().setKey("aws.sqs-event.akid").setValue(None)
                .setDesc("AKID for SQS connections"))
  citems.append(ConfigItem().setKey("aws.sqs-event.sak").setValue(None)
                .setDesc("Secret Access Key for SQS connections"))
  citems.append(ConfigItem().setKey("aws.sqs-event.region").setValue(None)
                .setDesc("Region for SQS Event Queue"))
  citems.append(ConfigItem().setKey("aws.sqs-event.url").setValue(None)
                .setDesc("URL for SQS Event Queue"))
  [ConfigDB.installConfigItem(x) for x in citems]

  akid = ConfigDB.getConfigItemByKey("aws.sqs-event.akid").getValue()
  if akid:
    sak = ConfigDB.getConfigItemByKey("aws.sqs-event.sak").getValue()
    region = ConfigDB.getConfigItemByKey("aws.sqs-event.region").getValue()
    url = ConfigDB.getConfigItemByKey("aws.sqs-event.url").getValue()
    Publisher.setup(akid, sak, region, url)
    Publisher.site_tag = ConfigDB.getConfigItemByKey("vts.geni.cm-tag").getValue()
    Publisher.publishEvent("VTS Startup", {})

