# Copyright (c) 2011-2012  The Board of Trustees of The Leland Stanford Junior University

import functools
import traceback

from ..core.log import KeyAdapter
from ..core.json import jsonify, JSONValidationError

def route (path, methods):
  def decorator (target):
    target.route = ((path, methods))
    return target
  return decorator

class Dispatcher(object):
  def __init__ (self, key, logger, app):
    self._log = KeyAdapter(key, logger)
    self._app = app

    for k in dir(self):
      try:
        val = getattr(self, k)
        (path, methods) = val.route
      except TypeError:
        continue
      except AttributeError:
        continue

      try:
        app.add_url_rule(path, path.replace("/", "_"), excwrapper(self._log, val), methods=methods)
        self._log.debug("Added route for: %s with method <%s>" % (path, k))
      except Exception, e:
        self._log.exception("Adding URL Rule")
        continue

def excwrapper (log, func):
  @functools.wraps(func)
  def wrap (*args, **kwargs):
    try:
      return func(*args, **kwargs)
    except JSONValidationError, e:
      jd = e.__json__()
      return jsonify(jd, code = 1, msg = jd["exception"])
    except Exception, e:
      log.exception("Exception")
      return jsonify(None, code = 2, msg  = traceback.format_exc())
  return wrap
