# Copyright (c) 2011-2012  The Board of Trustees of The Leland Stanford Junior University
# Copyright (c) 2012-2018  Barnstormer Softworks, Ltd.

import weakref
import logging

from flask import request

from foam.core.log import KeyAdapter
from foam.core.configdb import ConfigDB, ConfigItem, coerceBool
from foam.core.exception import CoreException

DISPATCHERS = []

def updateLogging (key, newval):
  newval = coerceBool(key, newval)
  for instance in DISPATCHERS:
    if instance() is not None:
      instance().updateLogging(newval)
  return newval
  
class Dispatcher(object):
  def __init__ (self, key, log):
    DISPATCHERS.append(weakref.ref(self))
    self._key = key
    self._log = KeyAdapter(key, log)
    self._logparams = ConfigDB.getConfigItemByKey("api.xmlrpc.log-params").getValue()

  def updateLogging (self, newval):
    self._logparams = newval

  def _dispatch (self, method, params):
    self._log.info("Request classified from: %s" % (request.environ["GEOIP_COUNTRY_CODE"]))
    self._log.info("Called: <%s>" % (method))
    if self._logparams:
      self._log.debug("Parameters: <%s>" % (str(params)))

    try:
      meth = getattr(self, "pub_%s" % (method))
    except AttributeError, e:
      self._log.warning("Client called unknown method: <%s>" % (method))
      return

    try:
      return meth(*params)
    except CoreException, x:
      x.log(self._log, "Call to known method <%s> failed!" % (method))
      raise Exception(str(x))
    except Exception, e:
      if not hasattr(e, "_foam_logged"):
        self._log.exception("Call to known method <%s> failed!" % (method))
      raise Exception(str(e))
 

def setup (app):
  citems = []
  citems.append(ConfigItem().setKey("api.xmlrpc.log-params").setValue(False)
                .setDesc("Log all parameters to XML-RPC calls (VERBOSE!)")
                .setUpdateFuncName("foam.api.xmlrpc.updateLogging"))
  [ConfigDB.installConfigItem(x) for x in citems]
