# Copyright (c) 2016  Barnstormer Softworks, Ltd.

from __future__ import absolute_import

from . import json
from . import util
from .configdb import ConfigDB, ConfigItem
from .log import KeyAdapter


class GAPIActionPolicy(object):
  class V2(object):
    LISTRESOURCES = "listresources"
    RENEW = "renewsliver"
    STATUS = "sliverstatus"
    DELETE = "deletesliver"

  class V3(object):
    DESCRIBE = "describe"
    ALLOCATE = "allocate"
    RENEW = "renew"
    PROVISION = "provision"
    STATUS = "status"
    POA = "performoperationalaction"
    DELETE = "delete"


  def __init__ (self):
    self._locked = set()

  def enable (self, privs):
    self._locked |= set(privs)

  def disable (self, privs):
    self._locked -= set(privs)

  def locked (self, priv):
    return priv in self._locked


class _AllocationPolicyManager(object):
  def __init__ (self):
    self._poldb = {}


GAPIActionPolicy.V2Actions = set([x[1] for x in util.pubvars(GAPIActionPolicy.V2)])
GAPIActionPolicy.V3Actions = set([x[1] for x in util.pubvars(GAPIActionPolicy.V3)])

def setup (app):
  citems = []
  citems.append(ConfigItem().setKey("core.policy-data").setValue({})
                .setDesc("Policy info data blob")
                .setUpdateFuncName("foam.core.configdb.checkDict"))
  [ConfigDB.installConfigItem(x) for x in citems]
