# Copyright (c) 2012  Barnstormer Softworks, Ltd.

from sqlalchemy import (Table, Column, MetaData, ForeignKey, create_engine, select, String,
                        Integer, Text, PickleType)
from sqlalchemy.orm import scoped_session, sessionmaker

from foam.core.configdb import ConfigDB

class _DB(object):
  def __init__ (self):
    self.db_path = ConfigDB.getConfigItemByKey("dhcp.dbpath").getValue()

    self.engine = create_engine("sqlite:///%s" % (self.db_path), pool_recycle=6000)
    self.metadata = MetaData()

    self.buildSchema()

    if not os.path.exists(self.db_path):
      self.__initDBCore()

    self.__hostCache = {}
    self.__interfaceCache = {}
    self.__subnetCache = {}
    self.__optionCache = {}
    self.__rangeCache = {}

  def __initDBCore (self):
    self.metadata.drop_all()
    self.metadata.create_all()

  def __buildSchema (self):
    self.tHosts = Table('hosts', self.metadata,
      Column('id', Integer, primary_key = True),
      Column('name', String(32)),
      Column('desc', Text))

    self.tInterfaces = Table('interfaces', self.metadata,
      Column('id', Integer, primary_key = True),
      Column('hostid', Integer, ForeignKey('hosts.id')),
      Column('name', String(32)),
      Column('hardware_address', String(64)),
      Column('ip_address', PickleType),
      Column('pxe_filename', String(256)),
      Column('next_server', String(256)),
      Column('pxe_enabled', Boolean, default=False))

    self.tSubnets = Table('subnets', self.metadata,
      Column('id', Integer, primary_key = True),
      Column('network', PickleType))  # IPy.IP

    self.tOptions = Table('options', self.metadata,
      Column('id', Integer, primary_key = True),
      Column('name', String(64)),
      Column('value', PickleType),
      Column('global', Boolean = True))

    self.jOptionsSubnets = Table('j_options_subnets', self.metadata,
      Column('options_id', Integer, ForeignKey("options.id")),
      Column('subnet_id', Integer, ForeignKey("subnets.id")))

    self.tRanges = Table('ranges', self.metadata,
      Column('id', Integer, primary_key = True),
      Column('subnetid', Integer, ForeignKey("subnets.id")),
      Column('start', PickleType),
      Column('end', PickleType),
      Column('dynamic_bootp', Boolean, default=False))

  def getHost (self, name = None, hostid = None):
    pass

  def getGlobalOptions (self):
    if len(self.__optionCache) == 0:
      conn = self.connection()
      s = select([self.tOptions], self.tOptions.c.global==True)
      res = conn.execute(s)

      for row in res:
        self.__optionCache[row[self.tOptions.c.name]] = row[self.tOptions.c.value]

    return [x for x in self.__optionCache.values() if x.global]

  def write (self):
    conflines = []

    opts = self.getGlobalOptions()
    for opt in opts:
      confline.append(str(opt))

    confline.append("")
    

DB = _DB()
