# Copyright (c) 2014  Barnstormer Softworks, Ltd.

from __future__ import absolute_import

from cStringIO import StringIO
import datetime

from lxml import etree as ET
from lxml import etree

from .db import GeniDB
from .lib import DuplicateSliver, RspecParseError, RSpecValidationError, GENISliver
from ..creds import CredVerifier
from ..lib import _asUTC
from . import approval
from ..core.configdb import ConfigDB
from .. import task


def oldCreateSliver (slice_urn, credentials, rspec, user_info):
  if GeniDB.sliceExists(slice_urn):
    raise DuplicateSliver(slice_urn)

  creds = CredVerifier.fromStrings(credentials, "createsliver", slice_urn)
  try:
    s = StringIO(rspec)
    rspec_dom = ET.parse(s)
  except Exception, exc:
    raise RspecParseError(slice_urn, str(exc))

  of3 = open("/opt/foam/schemas/of-resv-3.xsd", "r")
  xsdoc3 = etree.parse(of3)
  xs3 = etree.XMLSchema(xsdoc3)

  try:
    xs3.assertValid(rspec_dom)
  except etree.DocumentInvalid, e:
    raise RspecValidationError(str(e))

  rspec_elem = rspec_dom.getroot()
  schemas = rspec_elem.get("{%s}schemaLocation" % (XSNS))

  expiration = _asUTC(datetime.datetime.utcnow()) + ConfigDB.getConfigItemByKey("geni.max-lease").getValue()
  for cred in creds:
    credexp = _asUTC(cred.expiration)
    if credexp < expiration:
      expiration = credexp

  GeniDB.refreshDevices()

  sliver = GENISliver(rspec_dom)

  sliver.setUserURN(user_info["urn"])
  sliver.setUserEmail(user_info["email"], overwrite=False)
  sliver.validate()

  GeniDB.insertSliver(slice_urn, sliver, rspec, expiration)
  return sliver


def oldFoamCreateWithManifest (log, slice_urn, credentials, rspec, user_info):
  sliver = oldCreateSliver(slice_urn, credentials, rspec, user_info)
  auto_priority = ConfigDB.getConfigItemByKey("geni.auto-sliver-priority").getValue()

  approve = approval.analyzeForApproval(sliver)
  style = ConfigDB.getConfigItemByKey("geni.approval.approve-on-creation").getValue()
  if style == approval.NEVER:
    approve = False
  elif style == approval.ALWAYS:
    approve = True
  if approve:
    pid = task.approveSliver(sliver.getURN(), auto_priority)
    log.debug("task.py launched for approve-sliver (PID: %d)" % pid)

  data = GeniDB.getSliverData(sliver.getURN(), True)
  foam.task.emailCreateSliver(data)
  return GeniDB.getManifest(sliver.getURN())

