# Copyright (c) 2015-2016  Barnstormer Softworks, Ltd.

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

from pysnmp.hlapi import *

from . import pdu

class SNMPPDUDriver(pdu.PDUDriver):
  def __init__ (self, ip, port = 161, rostring = "public", rwstring = "private"):
    super(SNMPPDUDriver, self).__init__()
    self._ip = ip
    self._port = port
    self._rostring = rostring
    self._rwstring = rwstring
    self._pdu = UdpTransportTarget((ip, port))

  def _getValue (self, oid):
    g = getCmd(SnmpEngine(), CommunityData(self._rostring), self._pdu, ContextData(),
               ObjectType(ObjectIdentity(oid)))
    (errInd, errStatus, errIdx, varBinds) = next(g)
    return varBinds[0][1]._value

  def _setValue (self, oid, val):
    s = setCmd(SnmpEngine(), CommunityData(self._rwstring), self._pdu, ContextData(),
               ObjectType(ObjectIdentity(oid), val))
    (errInd, errStatus, errIdx, varBinds) = next(s)


class CPPort(pdu.PDUPort):
  def __init__ (self, pdu, num, name):
    super(CPPort, self).__init__(num, name)
    self._pdu = pdu

  def rename (self, name):
    self._pdu._renamePort(self, name)


class CyberPower(SNMPPDUDriver):
  NUM_OUTLETS = "1.3.6.1.4.1.3808.1.1.3.1.8.0"
  STATUS_NAME = "1.3.6.1.4.1.3808.1.1.3.3.5.1.1.2.%(port)d"
  CONFIG_NAME = "1.3.6.1.4.1.3808.1.1.3.3.4.1.1.2.%(port)d"

  def __init__ (self, ip, port = 161, rostring = "public", rwstring = "private"):
    super(CyberPower, self).__init__(ip, port, rostring, rwstring)

  @property
  def num_ports (self):
    if not self._numports:
      self._numports = self._getValue(CyberPower.NUM_OUTLETS)
    return self._numports

  @property
  def ports (self):
    if not self._ports:
      for port in xrange(1,self.num_ports+1):
        self._ports.append(CPPort(self, port, self._getValue(CyberPower.STATUS_NAME % ({"port" : port}))))
    for port in self._ports:
      yield port

  def _renamePort (self, port, name):
    self._setValue(CyberPower.CONFIG_NAME % ({"port" : port.num}), OctetString(name))
