# Copyright (c) 2016  Barnstormer Softworks

# Driver for integrating with Barnstormer dstat collector
# for docker container statistics

from __future__ import absolute_import

import json

from ...core.log import KeyAdapter

try:
  from dstat import config
  import zmq
except ImportError:
  config = None


class _Driver(object):
  def __init__ (self):
    if not config:
      return
    self.context = zmq.Context()
    self.socket = self.context.socket(zmq.REQ)
    self.socket.connect(config.CORE_SOCK)
    self._log = None

  @property
  def log (self):
    return self._log

  @log.setter
  def log (self, val):
    self._log = KeyAdapter("DSTAT", val)

  def _addContainer (self, cid, name):
    self.log.debug("Adding container: %s" % (cid))
    self.socket.send(json.dumps({"method" : "add-container", "container-id" : cid,
                                 "container-name" : name}))
    message = self.socket.recv()

  def _addGroup (self, name, container_ids):
    self.log.debug("Adding group: %s" % (name))
    self.socket.send(json.dumps({"method" : "add-group", "group-name" : name,
                                 "container-ids" : container_ids}))
    message = self.socket.recv()

  def _delGroup (self, name):
    self.socket.send(json.dumps({"method" : "del-group", "group-name" : name}))
    message = self.socket.recv()

  def _delContainer (self, cid):
    self.socket.send(json.dumps({"method" : "del-container", "container-id" : cid}))
    message = self.socket.recv()

  def addSliver (self, sliver):
    if not config:
      return
    self.log.info("Adding sliver: %s" % (sliver.urn))
    try:
      self._addGroup(sliver.urn, [x.longid for x in sliver.containers])
      for x in sliver.containers:
        self._addContainer(x.longid, x.client_id)
    except IndexError:
      # TODO: Report - we presume this is from loading containers that don't really exist
      return

  def delSliver (self, sliver):
    if not config:
      return
    self._delGroup(sliver.urn)
    for x in sliver.containers:
      try:
        self._delContainer(x.longid)
      except IndexError:
        continue


Driver = _Driver()
