# Copyright (c) 2016-2017  Barnstormer Softworks, Ltd.

from __future__ import absolute_import

import logging

from ...core.log import KeyAdapter
from ..drivers.docker import HostPathVolume

LOG = logging.getLogger("foam")

class ImageHandler(object):
  def __init__ (self, desc):
    self.log = KeyAdapter(desc.name, LOG)
    self.log.debug("Loaded")
    self.desc = desc
    self.attrs = {} # { <name> : <value> }
    for attr in self.desc.image_attributes.values():
      if attr.optional:
        self.attrs[attr.name] = attr.default
    self.volumes = []
    self.container = None

  def __json__ (self):
    d = {"version" : 1}
    d["attributes"] = []
    for name,val in self.attrs.items():
      # Sequences are serialized as comma separated values, we only handle one level deep for str()
      if isinstance(val, (list, tuple)):
        d["attributes"].append({"name" : name, "val" : ",".join([str(x) for x in val])})
      else:
        d["attributes"].append({"name" : name, "val" : str(val)})
    d["volumes"] = self.volumes
    return d

  # Event callbacks
  def prebuild (self, cobj):
    pass

  def postlocaltopo (self, cobj):
    pass

  def postnettopo (self, cobj):
    pass

  def postrm (self, cobj):
    pass

  # Common utilities
  # TODO: Should be filtered based on attributes, or in mixins, or something
  def setIPAddress (self, cobj, intf_name, addr, prefix_len):
    from ..drivers.docker import DockerDriver
    dd = DockerDriver(self.log.info)
    ipstr = "%s/%d" % (addr, prefix_len)
    dd.setInterfaceIP(cobj, intf_name, ipstr)

  @classmethod
  def jconstruct (cls, desc, obj):
    c = cls(desc)
    if obj["version"] == 1:
      for attr in obj["attributes"]:
        adesc = c.desc.image_attributes[attr["name"]]
        try:
          c.attrs[attr["name"]] = adesc.builder(attr["val"])
        except Exception:
          if not adesc.optional:
            LOG.warning("Unable to load non-optional attribute (%s)" % (attr["name"]))
          c.attrs[attr["name"]] = adesc.default
      for vol in obj["volumes"]:
        c.volumes.append(HostPathVolume(vol["hpath"], vol["cpath"]))
    return c

  @staticmethod
  def geniSetup (registry):
    pass
