import subprocess
import json
import time

INTF_DATA = {}
INTF_DELTAS = {}

OVSDB_CMD = r"""sudo ovsdb-client transact unix:/var/run/openvswitch/db.sock '["Open_vSwitch", {"op":"select", "table":"Interface", "where":[["type", "==", "patch"]], "columns":["name","statistics"]}]'"""

class StatEntry(object):
  def __init__ (self, name):
    self.name = name
    self.time = None
    self.rx_bytes = 0
    self.rx_packets = 0
    self.rx_dropped = 0
    self.rx_errors = 0
    self.tx_bytes = 0
    self.tx_packets = 0
    self.tx_errors = 0
    self.tx_dropped = 0

def gatherNetDev ():
  f = open("/proc/net/dev", "r")
  d = f.read()
  f.close()
  return d

def gatherPatch ():
  p = subprocess.Popen(OVSDB_CMD, stdout=subprocess.PIPE, shell=True)
  (sout, serr) = p.communicate()
  return sout


def parseNetDev (data):
  sentries = []
  for l in data.split('\n'):
    il = l.split()
    if len(il) != 17: # Exclude column headers and trailer
      continue
    se = StatsEntry(il[0])
    se.rx_bytes = il[1]
    se.rx_packets = il[2]
    se.rx_errors = il[3]
    se.rx_dropped = il[4]
    se.tx_bytes = il[9]
    se.tx_packets = il[10]
    se.tx_errors = il[11]
    se.tx_dropped = il[12]
    sentries.append(se)
  return sentries

def parsePatch (output):
  sentries = []
  data = json.loads(output)
  ilist = data[0]["rows"]
  for intf in ilist:
    se = StatsEntry(intf["name"])
    stats = intf["statistics"][1]
    for stat in stats:
      sname = stat[0]
      if sname == "rx_bytes":
        se.rx_bytes = stat[1]
      elif sname == "rx_packets":
        se.rx_packets = stat[1]
      elif sname == "rx_errors":
        se.rx_errors = stat[1]
      elif sname == "rx_dropped":
        se.rx_dropped = stat[1]
      elif sname == "tx_bytes":
        se.tx_bytes = stat[1]
      elif sname == "tx_dropped":
        se.tx_dropped = stat[1]
      elif sname == "tx_errors":
        se.tx_errors = stat[1]
      elif sname == "tx_packets":
        se.tx_packets = stat[1]
    sentries.append(se)
  return sentries

def main ():
  collect_loop()



if __name__ == '__main__':
  main()
